use crate::TokenKind;

#[derive(Debug, PartialEq)]
pub enum Expression {
    String(String),
    Number(f64),
    Bool(bool),
    Identifier(String),
    Prefix(Op, Box<Expression>),
    Infix(Box<Expression>, Op, Box<Expression>),
    Call(Box<Expression>, Vec<Expression>),
}

impl Expression {
    pub fn boxed(self) -> Box<Self> {
        Box::new(self)
    }
}

/// The `Op` enumeration is used to represent prefix, infix and other operations.
#[derive(Debug, PartialEq)]
pub enum Op {
    Add,
    Subtract,
    Multiply,
    Divide,
}

impl From<TokenKind> for Op {
    fn from(kind: TokenKind) -> Self {
        match kind {
            TokenKind::Plus => Op::Add,
            TokenKind::Minus => Op::Subtract,
            TokenKind::Asterisk => Op::Multiply,
            TokenKind::Slash => Op::Divide,
            _ => todo!()
        }
    }
}